@@ -86,7 +86,7 @@ class CallTracer(vm.VirtualMachine):
     kwargs.merge_instance_type_parameter(node, abstract_utils.V, value_type)
     return kwargs.to_variable(node)
 
-  def create_method_arguments(self, node, method):
+  def create_method_arguments(self, node, method, use_defaults=False):
     """Create arguments for the given method.
 
     Creates Unknown objects as arguments for the given method. Note that we
@@ -96,14 +96,29 @@ class CallTracer(vm.VirtualMachine):
     Args:
       node: The current node.
       method: An abstract.InterpreterFunction.
+      use_defaults: Whether to use parameter defaults for arguments. When True,
+        unknown arguments are created with force=False, as it is fine to use
+        Unsolvable rather than Unknown objects for type-checking defaults.
 
     Returns:
       A tuple of a node and a function.Args object.
     """
-    args = [self.convert.create_new_unknown(node, force=True)
-            for _ in range(method.argcount(node))]
-    kws = {key: self.convert.create_new_unknown(node, force=True)
-           for key in method.signature.kwonly_params}
+    args = []
+    num_posargs = method.argcount(node)
+    num_posargs_no_default = num_posargs - len(method.defaults)
+    for i in range(num_posargs):
+      default_idx = i - num_posargs_no_default
+      if use_defaults and default_idx >= 0:
+        arg = method.defaults[default_idx]
+      else:
+        arg = self.convert.create_new_unknown(node, force=not use_defaults)
+      args.append(arg)
+    kws = {}
+    for key in method.signature.kwonly_params:
+      if use_defaults and key in method.kw_defaults:
+        kws[key] = method.kw_defaults[key]
+      else:
+        kws[key] = self.convert.create_new_unknown(node, force=not use_defaults)
     starargs = self.create_varargs(node) if method.has_varargs() else None
     starstarargs = self.create_kwargs(node) if method.has_kwargs() else None
     return node, function.Args(posargs=tuple(args),
@@ -138,7 +153,17 @@ class CallTracer(vm.VirtualMachine):
     self.pop_frame(frame)
     return state.node, ret
 
-  def maybe_analyze_method(self, node, val):
+  def _maybe_fix_classmethod_cls_arg(self, node, cls, func, args):
+    sig = func.signature
+    if (args.posargs and sig.param_names and
+        (sig.param_names[0] not in sig.annotations)):
+      # fix "cls" parameter
+      return args._replace(
+          posargs=(cls.AssignToNewVariable(node),) + args.posargs[1:])
+    else:
+      return args
+
+  def maybe_analyze_method(self, node, val, cls=None):
     method = val.data
     fname = val.data.name
     if isinstance(method, abstract.INTERPRETER_FUNCTION_TYPES):
@@ -191,18 +216,19 @@ class CallTracer(vm.VirtualMachine):
     log.info("Unable to generate fake arguments for %s", funcv)
     return node, self.new_unsolvable(node)
 
-  def analyze_method_var(self, node0, name, var):
+  def analyze_method_var(self, node0, name, var, cls=None):
     log.info("Analyzing %s", name)
     node1 = node0.ConnectNew(name)
     for val in var.bindings:
-      node2 = self.maybe_analyze_method(node1, val)
+      node2 = self.maybe_analyze_method(node1, val, cls)
       node2.ConnectTo(node0)
     return node0
 
   def bind_method(self, node, name, methodvar, instance_var):
     bound = self.program.NewVariable()
     for m in methodvar.Data(node):
-      bound.AddBinding(m.property_get(instance_var), [], node)
+      is_cls = False
+      bound.AddBinding(m.property_get(instance_var, is_cls), [], node)
     return bound
 
   def _instantiate_binding(self, node0, cls):
@@ -218,11 +244,7 @@ class CallTracer(vm.VirtualMachine):
     for b in new.bindings:
       self._analyzed_functions.add(b.data.get_first_opcode())
       node2, args = self.create_method_arguments(node1, b.data)
-      if args.posargs and (
-          b.data.signature.param_names[0] not in b.data.signature.annotations):
-        # fix "cls" parameter
-        args = args._replace(
-            posargs=(cls.AssignToNewVariable(node0),) + args.posargs[1:])
+      args = self._maybe_fix_classmethod_cls_arg(node0, cls, b.data, args)
       node3 = node2.ConnectNew()
       node4, ret = self.call_function_with_args(node3, b, args)
       instance.PasteVariable(ret)
@@ -318,7 +340,6 @@ class CallTracer(vm.VirtualMachine):
 
   def call_init(self, node, instance):
     # Call __init__ on each binding.
-    # TODO(kramm): This should do join_cfg_nodes, instead of concatenating them.
     for b in instance.bindings:
       if b.data in self._initialized_instances:
         continue
@@ -345,7 +366,7 @@ class CallTracer(vm.VirtualMachine):
       if name in self._CONSTRUCTORS:
         continue  # We already called this method during initialization.
       b = self.bind_method(node, name, methodvar, instance)
-      node = self.analyze_method_var(node, name, b)
+      node = self.analyze_method_var(node, name, b, val)
     return node
 
   def analyze_function(self, node0, val):
@@ -457,14 +478,18 @@ class CallTracer(vm.VirtualMachine):
     return classes
 
   def pytd_for_types(self, defs):
+    # If a variable is annotated, we'll always output that type.
+    annotated_names = set()
     data = []
     pytd_convert = self.convert.pytd_convert
-    annots = abstract_utils.get_annotations_dict(defs) or {}
-    for name, t in pytd_convert.uninitialized_annotations_to_instance_types(
-        self.exitpoint, annots, defs):
+    annots = abstract_utils.get_annotations_dict(defs)
+    for name, t in pytd_convert.annotations_to_instance_types(
+        self.exitpoint, annots):
+      annotated_names.add(name)
       data.append(pytd.Constant(name, t))
     for name, var in defs.items():
-      if name in output.TOP_LEVEL_IGNORE or self._is_builtin(name, var.data):
+      if (name in output.TOP_LEVEL_IGNORE or name in annotated_names or
+          self._is_builtin(name, var.data)):
         continue
       options = var.FilteredData(self.exitpoint, strict=False)
       if (len(options) > 1 and
@@ -505,7 +530,6 @@ class CallTracer(vm.VirtualMachine):
           arg_names[i] = function.argname(i)
       arg_types = (a.data.to_type(node) for a in args)
       ret = pytd_utils.JoinTypes(t.to_type(node) for t in retvar.data)
-      # TODO(kramm): Record these:
       starargs = None
       starstarargs = None
       funcs[func.data.name].add(pytd.Signature(
@@ -557,9 +581,6 @@ class CallTracer(vm.VirtualMachine):
       ))
     return classes
 
-  def pytd_aliases(self):
-    return ()  # TODO(kramm): Compute these.
-
   def pytd_classes_for_namedtuple_instances(self):
     return tuple(v.generate_ast() for v in self._generated_classes.values())
 
@@ -568,7 +589,7 @@ class CallTracer(vm.VirtualMachine):
                tuple(self.pytd_classes_for_call_traces()) +
                self.pytd_classes_for_namedtuple_instances())
     functions = tuple(self.pytd_functions_for_call_traces())
-    aliases = tuple(self.pytd_aliases())
+    aliases = ()  # aliases are instead recorded as constants
     ty = pytd_utils.Concat(
         self.pytd_for_types(defs),
         pytd_utils.CreateModule("unknowns", classes=classes,
