@@ -186,6 +186,12 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
       annotations[name] = annot or self.vm.convert.unsolvable
     return annotations
 
+  def init_annotation(self, node, name, annot, extra_key=None):
+    node, value = self.vm.init_class(node, annot, extra_key=extra_key)
+    for d in value.data:
+      d.from_annotation = name
+    return node, value
+
   def apply_annotation(self, state, op, name, value):
     """If there is an annotation for the op, return its value."""
     assert op is self.vm.frame.current_opcode
@@ -202,7 +208,7 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
           self.vm.frames, annot, details=errorlog.details)
     typ = self.extract_annotation(
         state.node, var, name, self.vm.simple_stack(), is_var=True)
-    _, value = self.vm.init_class(state.node, typ)
+    _, value = self.init_annotation(state.node, name, typ)
     return typ, value
 
   def extract_annotation(self, node, var, name, stack, is_var=False):
@@ -220,22 +226,6 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
       return self.vm.convert.unsolvable
     return typ
 
-  def init_from_annotations(self, node, name, annots_var):
-    """Instantiate `name` from the given annotations dict, calling __init__."""
-    try:
-      annots = abstract_utils.get_atomic_python_constant(annots_var, dict)
-    except abstract_utils.ConversionError:
-      return None
-    if name not in annots:
-      return None
-    try:
-      typ = abstract_utils.get_atomic_value(annots[name])
-    except abstract_utils.ConversionError:
-      self.vm.errorlog.ambiguous_annotation(self.vm.frames, None, name)
-      return self.vm.new_unsolvable(node)
-    _, value = self.vm.init_class(node, typ)
-    return value
-
   def eval_multi_arg_annotation(self, node, func, annot, stack):
     """Evaluate annotation for multiple arguments (from a type comment)."""
     args, errorlog = self._eval_expr_as_tuple(node, annot, stack)
