@@ -9,6 +9,7 @@ from pytype import abstract_utils
 from pytype import mixin
 from pytype import special_builtins
 from pytype import utils
+from pytype.overlays import dataclass_overlay
 from pytype.overlays import typing_overlay
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
@@ -216,13 +217,19 @@ class Converter(utils.VirtualMachineWeakrefMixin):
         return pytd.AnythingType()
     elif isinstance(v, typing_overlay.TypeVar):
       return pytd.NamedType("__builtin__.type")
+    elif isinstance(v, dataclass_overlay.FieldInstance):
+      if not v.default:
+        return pytd.AnythingType()
+      return pytd_utils.JoinTypes(
+          self.value_to_pytd_type(node, d, seen, view)
+          for d in v.default.data)
     elif isinstance(v, abstract.FUNCTION_TYPES):
       try:
         signatures = abstract_utils.get_signatures(v)
       except NotImplementedError:
         return pytd.NamedType("typing.Callable")
       if len(signatures) == 1:
-        val = self.signature_to_callable(signatures[0], self.vm)
+        val = self.signature_to_callable(signatures[0])
         if not isinstance(v, abstract.PYTD_FUNCTION_TYPES) or not val.formal:
           # This is a workaround to make sure we don't put unexpected type
           # parameters in call traces.
@@ -275,37 +282,36 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     else:
       raise NotImplementedError(v.__class__.__name__)
 
-  def signature_to_callable(self, sig, vm):
+  def signature_to_callable(self, sig):
     """Converts a function.Signature object into a callable object.
 
     Args:
       sig: The signature to convert.
-      vm: The vm instance.
 
     Returns:
       An abstract.CallableClass representing the signature, or an
       abstract.ParameterizedClass if the signature has a variable number of
       arguments.
     """
-    base_cls = vm.convert.function_type
-    ret = sig.annotations.get("return", vm.convert.unsolvable)
+    base_cls = self.vm.convert.function_type
+    ret = sig.annotations.get("return", self.vm.convert.unsolvable)
     if self._detailed or (
         sig.mandatory_param_count() == sig.maximum_param_count()):
       # If self._detailed is false, we throw away the argument types if the
       # function takes a variable number of arguments, which is correct for pyi
       # generation but undesirable for, say, error message printing.
-      args = [sig.annotations.get(name, vm.convert.unsolvable)
+      args = [sig.annotations.get(name, self.vm.convert.unsolvable)
               for name in sig.param_names]
-      params = {abstract_utils.ARGS: vm.merge_values(args),
+      params = {abstract_utils.ARGS: self.vm.merge_values(args),
                 abstract_utils.RET: ret}
       params.update(enumerate(args))
-      return abstract.CallableClass(base_cls, params, vm)
+      return abstract.CallableClass(base_cls, params, self.vm)
     else:
       # The only way to indicate a variable number of arguments in a Callable
       # is to not specify argument types at all.
-      params = {abstract_utils.ARGS: vm.convert.unsolvable,
+      params = {abstract_utils.ARGS: self.vm.convert.unsolvable,
                 abstract_utils.RET: ret}
-      return abstract.ParameterizedClass(base_cls, params, vm)
+      return abstract.ParameterizedClass(base_cls, params, self.vm)
 
   def value_to_pytd_def(self, node, v, name):
     """Get a PyTD definition for this object.
@@ -356,12 +362,15 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     else:
       raise NotImplementedError(v.__class__.__name__)
 
-  def uninitialized_annotations_to_instance_types(self, node, annots, members):
+  def annotations_to_instance_types(self, node, annots):
     """Get instance types for annotations not present in the members map."""
-    for name in annots:
-      if name not in members:
-        yield name, pytd_utils.JoinTypes(
-            value.get_instance_type(node) for value in annots[name].data)
+    if annots:
+      for name, typ in annots.get_annotations(node):
+        contained_type = abstract_utils.match_type_container(
+            typ, "typing.ClassVar")
+        if contained_type:
+          typ = contained_type
+        yield name, typ.get_instance_type(node)
 
   def _function_call_to_return_type(self, node, v, seen_return, num_returns):
     """Get a function call's pytd return type."""
@@ -419,7 +428,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
         starargs=starargs,
         starstarargs=starstarargs,
         return_type=ret,
-        exceptions=(),  # TODO(kramm): record exceptions
+        exceptions=(),  # TODO(b/159052087): record exceptions
         template=())
 
   def _function_to_def(self, node, v, function_name):
@@ -518,15 +527,16 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     methods = {}
     constants = collections.defaultdict(pytd_utils.TypeBuilder)
 
-    annots = abstract_utils.get_annotations_dict(v.members) or {}
+    annots = abstract_utils.get_annotations_dict(v.members)
+    annotated_names = set()
 
-    for name, t in self.uninitialized_annotations_to_instance_types(
-        node, annots, v.members):
+    for name, t in self.annotations_to_instance_types(node, annots):
       constants[name].add_type(t)
+      annotated_names.add(name)
 
     # class-level attributes
     for name, member in v.members.items():
-      if name in CLASS_LEVEL_IGNORE:
+      if name in CLASS_LEVEL_IGNORE or name in annotated_names:
         continue
       for value in member.FilteredData(self.vm.exitpoint, strict=False):
         if isinstance(value, special_builtins.PropertyInstance):
@@ -572,9 +582,10 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     # instance-level attributes
     for instance in set(v.instances):
       for name, member in instance.members.items():
-        if name not in CLASS_LEVEL_IGNORE:
-          for value in member.FilteredData(self.vm.exitpoint, strict=False):
-            constants[name].add_type(value.to_type(node))
+        if name in CLASS_LEVEL_IGNORE or name in annotated_names:
+          continue
+        for value in member.FilteredData(self.vm.exitpoint, strict=False):
+          constants[name].add_type(value.to_type(node))
 
     for name in list(methods):
       if name in constants:
