@@ -130,8 +130,6 @@ class ParserTest(_ParserTestBase):
        y""", 3, "Invalid indentation")
 
   def test_type_on_next_line(self):
-    # TODO(dbaum): This probably should be an error.  Current behavior matches
-    # legacy parser. Consider changing to an error.
     self.check("""
       a = ...
       # type: int""",
@@ -1014,6 +1012,12 @@ class FunctionTest(_ParserTestBase):
       class Foo:
           bar: str
     """)
+    self.check("""
+      def f(  # type: ignore
+        x: int) -> None: ...
+    """, """
+      def f(x: int) -> None: ...
+    """)
 
   def test_decorators(self):
     # These tests are a bit questionable because most of the decorators only
