@@ -143,7 +143,6 @@ class Next(BuiltinFunction):
       ret.PasteVariable(default)
       return node, ret
     else:
-      # TODO(kramm): This needs a test case.
       return node, self.vm.new_unsolvable(node)
 
 
@@ -797,4 +796,6 @@ class ClassMethod(BuiltinClass):
     if len(args.posargs) != 1:
       raise function.WrongArgCount(self._SIGNATURE, args, self.vm)
     arg = args.posargs[0]
+    for d in arg.data:
+      d.is_classmethod = True
     return node, ClassMethodInstance(self.vm, self, arg).to_variable(node)
